'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _eos = require('./eos');

var eos = _interopRequireWildcard(_eos);

var _Identity = require('../models/Identity');

var _Identity2 = _interopRequireDefault(_Identity);

var _ClaimReservation = require('../models/ClaimReservation');

var _ClaimReservation2 = _interopRequireDefault(_ClaimReservation);

var _murmurhash = require('murmurhash');

var _murmurhash2 = _interopRequireDefault(_murmurhash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const fingerprinted = username => _murmurhash2.default.v2(username.toLowerCase());

const getIdentity = (() => {
    var _ref = _asyncToGenerator(function* (username) {
        return yield eos.read({
            table: 'ids',
            index: fingerprinted(username),
            limit: 1,
            model: _Identity2.default,
            firstOnly: true
        });
    });

    return function getIdentity(_x) {
        return _ref.apply(this, arguments);
    };
})();

class IdentityService {

    constructor() {
        this.paymentSymbol = 'EOS';
    }

    setSymbol(symbol) {
        this.paymentSymbol = symbol;
    }

    validName(name) {
        return (/^[a-zA-Z0-9_-]{3,20}$/.test(name)
        );
    }

    get(name) {
        return _asyncToGenerator(function* () {
            return getIdentity(name);
        })();
    }

    payAndIdentify(username, key) {
        var _this = this;

        return _asyncToGenerator(function* () {
            return eos.writer.transaction(['eosio.token', 'ridlridlridl'], function (contracts) {
                contracts.eosio_token.transfer(eos.account.name, 'ridlridlridl', `1.0000 ${_this.paymentSymbol}`, '', eos.options);
                contracts.ridlridlridl.identify(eos.account.name, username, key, eos.options);
            });
        })();
    }

    claim(username, key, signature) {
        var _this2 = this;

        return _asyncToGenerator(function* () {
            return eos.contract.claim(eos.account.name, username, key, signature, eos.options).then(function () {
                return _this2.get(username);
            }).catch(function (err) {
                return console.error(err);
            });
        })();
    }

    rekey(username, key) {
        var _this3 = this;

        return _asyncToGenerator(function* () {
            return eos.contract.rekey(username, key, eos.options).then(function () {
                return _this3.get(username);
            }).catch(function (err) {
                return false;
            });
        })();
    }

    setAccountFromAccount(username, account) {
        var _this4 = this;

        return _asyncToGenerator(function* () {
            return eos.contract.setaccacc(username, account, eos.options).then(function () {
                return _this4.get(username);
            }).catch(function (err) {
                return false;
            });
        })();
    }

    setAccountFromKey(username, account, signature) {
        var _this5 = this;

        return _asyncToGenerator(function* () {
            return eos.contract.setacckey(username, account, signature, eos.options).then(function () {
                return _this5.get(username);
            }).catch(function (err) {
                return false;
            });
        })();
    }

    release(username, signedHash) {
        return _asyncToGenerator(function* () {
            return eos.contract.release(username, signedHash, eos.options).then(function () {
                return true;
            }).catch(function (err) {
                return false;
            });
        })();
    }

    loadTokens(username, amount) {
        var _this6 = this;

        return _asyncToGenerator(function* () {
            amount = parseFloat(amount.toString().split(' ')[0]).toFixed(4);
            if (amount <= 0) throw new Error("Amount must be greater than 0");

            return eos.writer.transaction(['ridlridlcoin', 'ridlridlridl'], function (contracts) {
                contracts.ridlridlcoin.transfer(eos.account.name, 'ridlridlridl', `${amount} RIDL`, '', eos.options);
                contracts.ridlridlridl.loadtokens(eos.account.name, username, `${amount} RIDL`, eos.options);
            }).then(function () {
                return _this6.get(username);
            }).catch(function (err) {
                return false;
            });
        })();
    }

    identityBalance(username) {
        return _asyncToGenerator(function* () {
            return getIdentity(username).then(function (res) {
                return res.tokens;
            }).catch(function () {
                return null;
            });
        })();
    }

    getHash(name) {
        var _this7 = this;

        return _asyncToGenerator(function* () {
            return yield _this7.get(name).then(function (x) {
                return x ? x.hash : null;
            });
        })();
    }
    exists(name) {
        var _this8 = this;

        return _asyncToGenerator(function* () {
            return yield _this8.get(name).then(function (x) {
                return !!x;
            });
        })();
    }
}
exports.default = IdentityService;